package com.kexun.utils;

import com.alibaba.fastjson.annotation.JSONField;
import com.kexun.annotation.CrXmlSign;
import com.kexun.model.xml.IXmlBase;
import lombok.extern.slf4j.Slf4j;
import org.dom4j.Element;
import org.springframework.util.StringUtils;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Description: 反射工具类
 *
 * @author lgs
 * @date 2023/05/30
 */
@Slf4j
public class ReflexUtils {

    /**
     * 通过反射对目标类字段进行赋值
     *
     * @param nodeElement   xml节点
     * @param reportXmlBase 目标类
     * @throws Exception 异常信息
     */
    public static void setValue(Element nodeElement, IXmlBase reportXmlBase) throws Exception {
        Class<? extends IXmlBase> objectClass = reportXmlBase.getClass();
        Field[] declaredFields = objectClass.getDeclaredFields();
        for (Field field : declaredFields) {
            if (field.isAnnotationPresent(CrXmlSign.class)) {
                String xmlNodeName = field.getAnnotation(CrXmlSign.class).xmlNodeName();
                String xmlType = field.getAnnotation(CrXmlSign.class).xmlType();
                if (!StringUtils.isEmpty(xmlNodeName)) {
                    Element element = nodeElement.element(xmlNodeName);
                    if (element != null) {
                        field.setAccessible(true);
                        String text = element.getText();
                        if (xmlType.equals("Date") && text.contains("T")) {
                            text = DateUtils.convertTDate(text);
                        }
                        field.set(reportXmlBase, text);
                    } else {
                        log.info("根据报文节点名称{},xml报文里获取不到相关的值", xmlNodeName);
                    }
                }
            } else {
                log.info("该字段没有标记CrXmlSign注解，不进行处理");
            }
        }
    }

    /**
     * 对xml节点是集合的数据进行处理
     *
     * @param nodeElement 节点元素
     * @param object      实体类
     * @return 实体类集合
     * @throws Exception
     */
    public static List<IXmlBase> setListValue(Element nodeElement, Class<? extends IXmlBase> object) throws Exception {

        List xmlBases = new ArrayList<IXmlBase>();
        //得到这个信息节点的节点信息
        List<Element> elements = nodeElement.elements();
        for (Element element : elements) {
            List<Element> subElement = element.elements();
            //实例化信息类
            IXmlBase reportXmlBase = object.newInstance();
            Class<? extends IXmlBase> objectClass = reportXmlBase.getClass();
            Field[] declaredFields = objectClass.getDeclaredFields();
            //获取字段注解和字段名对应的map集合
            Map<String, String> fieldMap = new HashMap<>();
            for (Field field : declaredFields) {
                if (field.isAnnotationPresent(CrXmlSign.class)) {
                    String xmlNodeName = field.getAnnotation(CrXmlSign.class).xmlNodeName();
                    String name = field.getName();
                    fieldMap.put(xmlNodeName, name);
                }
            }
            //对类中的属性进行赋值
            for (Element subNode : subElement) {
                String name = subNode.getName();
                String text = subNode.getText();
                Field field = objectClass.getField(fieldMap.get(name));
                field.setAccessible(true);
                field.set(objectClass, text);
            }
            xmlBases.add(reportXmlBase);
        }
        return xmlBases;
    }

    /**
     * 判断某类中是否存在某字段
     *
     * @param clzz      类
     * @param fieldName 字段名
     * @return 是否
     */
    public static boolean hasField(Class clzz, String fieldName) {
        // 从当前类中查找
        Field declaredField = null;
        try {
            declaredField = clzz.getDeclaredField(fieldName);
        } catch (NoSuchFieldException e) {

        }
        if (null != declaredField) {
            return true;
        }
        // 从父类中查找 -- 仅限public方法
        Field field = null;
        try {
            field = clzz.getField(fieldName);
        } catch (NoSuchFieldException e) {

        }
        if (null != field) {
            return true;
        }
        return false;
    }

    /**
     * 获取@JSONField 注解name为
     *
     * @param obj
     * @param annotaionFiledName
     * @return
     */
    public static String getAnnotaionFiledValue(Object obj, String annotaionFiledName) {
        String value = "";
        if (obj != null) {
            Class<?> aClass = obj.getClass();
            Field[] declaredFields = aClass.getDeclaredFields();
            for (Field declaredField : declaredFields) {
                declaredField.setAccessible(true);
                JSONField annotation = declaredField.getAnnotation(JSONField.class);
                if (annotation != null) {
                    String annotaionName = annotation.name();
                    if (annotaionName.equals(annotaionFiledName)) {
                        try {
                            value = String.valueOf(declaredField.get(obj));
                            break;
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                        }
                    }
                }
            }
        }
        return value;
    }

}
